@@ -1,15 +1,20 @@
 import json
 import os
+import re
 import operator as _operator
 from pawnlib.config import pawnlib_config as pawn, pconf
 from pawnlib.typing import is_hex, is_int, is_float, FlatDict, Namespace, sys_exit, is_valid_token_address, is_valid_private_key
 from pawnlib.output.file import get_file_list
+from pawnlib.output.color_print import bcolors
 from InquirerPy import prompt, inquirer, get_style
 from InquirerPy.validator import NumberValidator
 from prompt_toolkit.validation import ValidationError, Validator, DynamicValidator
 from prompt_toolkit.shortcuts import prompt as toolkit_prompt
 import string
-from typing import Callable
+from typing import Callable, Optional
+import argparse
+from rich.console import Console
+
 all_special_characters = string.punctuation
 
 try:
@@ -17,7 +22,8 @@ try:
 except ImportError:
     from typing_extensions import Literal
 
-
+# from pawnlib.config.globalconfig import Null
+# pawn.console.debug = Null
 class PromptWithArgument:
     def __init__(self, argument="", min_length=1, max_length=1000, verbose=1,  **kwargs):
         self.argument = argument
@@ -54,7 +60,11 @@ class PromptWithArgument:
             ),
             select=dict(
                 # pointer="➡️ ",
-                pointer=" 👉"
+                pointer=" 👉",
+            ),
+            fuzzy=dict(
+                match_exact=True,
+                pointer="➤",
             )
         )
 
@@ -112,7 +122,7 @@ class PromptWithArgument:
             self._args = pconf().data.args
         except Exception as e:
             self._args = None
-            pawn.console.debug(f"[yellow] Error parsing - {e}")
+            # pawn.console.debug(f"[yellow] Error parsing - {e}")
 
         try:
             category = f"[{self._args.subparser_name.upper()}]"
@@ -171,12 +181,16 @@ class PromptWithArgument:
                     del self._options[param]
 
     def _push_argument(self, result="", target_args=None):
+        # if not target_args:
+        #     target_args = pconf().data.args
+        # setattr(target_args, self.argument, result)
         try:
             if not target_args:
                 target_args = pconf().data.args
             setattr(target_args, self.argument, result)
         except:
-            pawn.console.debug(f"args not found, {target_args}, {self.argument}, {result}")
+            pass
+            # pawn.console.debug(f"args not found, {target_args}, {self.argument}, {result}")
 
     def _skip_if_value_in_args(self):
         if self.argument_value:
@@ -201,8 +215,8 @@ class PromptWithArgument:
                         _argument_value = f"{len(self.argument_value)*'*'}"
                     else:
                         _argument_value = self.argument_value
-                    pawn.console.debug(f"Skipped [yellow]{self._options.get('name','')}[/yellow]"
-                                     f"prompt cause args value exists. --{self.argument_real_name}={_argument_value}")
+                    # pawn.console.debug(f"Skipped [yellow]{self._options.get('name','')}[/yellow]"
+                    #                  f"prompt cause args value exists. --{self.argument_real_name}={_argument_value}")
                 self._check_force_validation(name=self.argument_real_name, value=self.argument_value)
                 result = self._check_force_filtering(name=self.argument_real_name, value=self.argument_value)
             return result
@@ -223,7 +237,7 @@ class PromptWithArgument:
         try:
             _filtering = self._options.get('filter')
             if _filtering:
-                pawn.console.debug(f"force filtering name={name} value={value} return={_filtering(value)}")
+                # pawn.console.debug(f"force filtering name={name} value={value} return={_filtering(value)}")
                 return _filtering(value)
         except ValidationError as e:
             sys_exit(f"[bold]Failed to filtering : {e}")
@@ -239,7 +253,7 @@ class PromptWithArgument:
             # self._push_argument(result=input_value)
             return input_value
         else:
-            pawn.console.debug(f"Skipped prompt, {self.argument_name}={self.argument_value}")
+            # pawn.console.debug(f"Skipped prompt, {self.argument_name}={self.argument_value}")
             return False
 
     @_common_executes_decorator
@@ -252,7 +266,6 @@ class PromptWithArgument:
             self._set_style(style_type="prompt")
             if self._options.get('style'):
                 style = self._options.pop('style')
-            pawn.console.log(f"self._options={self._options},  style={style}")
             answer = prompt(questions=self._options, style=style, style_override=False)
             if answer.get('name'):
                 return answer['name']
@@ -982,19 +995,20 @@ def fuzzy_prompt_to_argument(**kwargs):
     if kwargs.get("argument"):
         argument = kwargs.pop("argument")
         _argument_name = argument.replace("_", "-")
-        _argument_value = pawn.data.args.__dict__.get(_argument_name, _arg_missing_word)
-
+        _argument_value = pawn.data.args.__dict__.get(argument, _arg_missing_word)
         if _arg_missing_word == _argument_value:
-            raise ValueError(f"Can not find argument in args. {argument}")
-        _default_value = kwargs.get('default')
+            raise ValueError(f"\n\n\nCould not find the argument in args."
+                             f" argument:'{argument}' , "
+                             f"_argument_name: '{_argument_name}',"
+                             f" _argument_value: '{_argument_value}'"
+                             )
+        _default_value = kwargs.get('default','')
 
         kwargs['instruction'] = kwargs.get('instruction', "") + f"( --{_argument_name} {_default_value})"
         if not _argument_value and argument:
             response_value = fuzzy_prompt(**kwargs)
             setattr(_pconf.data.args, argument, response_value)
             return response_value
-        else:
-            pawn.console.debug("")
     else:
         raise ValueError(f"Required argument, {kwargs}")
 
@@ -1154,3 +1168,221 @@ def json_input_prompt(default={}, message="Edit Transaction JSON"):
         validate=JsonValidator(),
     )
     return json.loads(json_text)
+
+
+class NewlineHelpFormatter(argparse.HelpFormatter):
+    def _fill_text(self, text, width, indent):
+        # 각 줄을 분리하여 리스트로 생성
+        lines = text.splitlines()
+        formatted_lines = []
+        for line in lines:
+            if line.strip() == '':
+                formatted_lines.append('')
+            else:
+                indented_line = '{}{}'.format(indent, line)
+                formatted_lines.append(indented_line)
+
+        formatted_text = '\n'.join(formatted_lines)
+        return formatted_text
+
+
+class ColoredHelpFormatter(argparse.RawDescriptionHelpFormatter):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._original_epilog = kwargs.get('epilog', '')
+        self.is_multi_line = True
+
+    def force_adjust_parts(self, parts, _sep_space):
+        combined_string = ''.join(parts)
+        # Find the position of the second space
+        second_space_pos = combined_string.find(_sep_space, combined_string.find(' ') + 1)
+
+        argument = combined_string[:second_space_pos]
+        description = f"{combined_string[second_space_pos+1:].strip()}\n"
+
+        parts = [argument, description]
+        self.is_multi_line = False
+        pawn.console.log(f"[blue]{parts}")
+        return parts
+
+
+    def _format_action(self, action):
+        formatted_action = super()._format_action(action)
+        _sep_space = " " * 3
+
+        parts = formatted_action.split('\n', 1)
+        # pawn.console.log(parts)
+        self.is_multi_line = True
+
+        # parts = self.force_adjust_parts(parts, _sep_space)
+        # pawn.console.log(parts)
+        if len(parts) == 2 and not parts[-1]:
+            parts = formatted_action.split(_sep_space, 1)
+            self.is_multi_line = False
+
+        parts[0] = bcolors.OKBLUE + parts[0] + bcolors.RESET
+        if len(parts) > 1:
+            parts[1] = bcolors.CYAN + parts[1] + bcolors.RESET
+
+        if self.is_multi_line:
+            return '\n'.join(parts)
+        else:
+            return _sep_space.join(parts)
+
+    def format_help(self):
+        help_text = bcolors.RESET + super().format_help()
+        epilog_prefix = "Usage examples:"
+
+        if epilog_prefix in help_text:
+            epilog_content_indicator = f"{epilog_prefix}\n"
+            self._original_epilog = f"{help_text.split(epilog_content_indicator, 1)[1]}"
+            epilog_prefix_title = bcolors.UNDERLINE + f"{epilog_prefix}" +bcolors.RESET
+            help_text = re.sub(rf'{epilog_prefix}(.|\n)*', f'{epilog_prefix_title}', help_text)
+
+        formatted_epilog_lines = []
+        for line in self._original_epilog.split('\n'):
+            if line.strip().startswith("pawns "):
+                formatted_epilog_lines.append(bcolors.OKBLUE + line + bcolors.RESET)
+            else:
+                formatted_epilog_lines.append(line)
+
+        formatted_epilog = '\n'.join(formatted_epilog_lines)
+        return help_text + '\n\n' + formatted_epilog
+
+class CustomArgumentParser(argparse.ArgumentParser):
+    def error(self, message):
+        self.print_help()
+        sys_exit(message, 2)
+
+
+def get_default_arguments(parser=None) -> dict:
+    """
+    Extract all default values from an ArgumentParser, including any subparsers.
+
+    :param parser: The ArgumentParser instance to extract defaults from.
+                   If None, an exception will be raised.
+    :return: A dictionary containing argument names as keys and their default values.
+    :raises ValueError: If no parser is provided.
+
+    This function iterates through the main parser's actions to collect default values.
+    If the parser has subparsers, it recursively collects their defaults as well.
+
+        Example:
+
+        .. code-block:: python
+
+            from pawnlib.input import get_default_arguments
+
+            parser = get_parser()
+            defaults = get_default_arguments(parser)
+            pawn.console.log(defaults)
+            # >> {'command': None, 'file': ['/var/log/system.log'], 'base_dir': '.', 'log_type': 'console', 'log_file': None}
+
+    """
+    if not isinstance(parser, argparse.ArgumentParser):
+        raise ValueError("Invalid parser. Please provide a valid ArgumentParser instance.")
+
+    defaults = {}
+    for action in parser._actions:
+        if action.dest != 'help' and action.default != argparse.SUPPRESS:
+            defaults[action.dest] = action.default
+
+    subparsers = getattr(parser, '_subparsers', None)
+    if subparsers:
+        for subparser_action in subparsers._group_actions:
+            for choice, subparser in subparser_action.choices.items():
+                sub_defaults = get_default_arguments(subparser)
+                defaults.update(sub_defaults)
+
+    return defaults
+
+
+def get_service_specific_arguments(parser=None, service_name=None):
+    """
+    Extracts the arguments for a specific service from the argparse parser.
+
+    :param parser: The top-level argparse parser.
+    :param service_name: The name of the service to extract arguments for ('ssh' or 'wallet').
+    :return: Dictionary of argument names and their default values for the specified service.
+    """
+    if not isinstance(parser, argparse.ArgumentParser):
+        raise ValueError("Invalid parser. Please provide a valid ArgumentParser instance.")
+
+    if not service_name:
+        raise ValueError("Invalid service_name. Please provide a service_name.")
+
+    subparsers_actions = [
+        action for action in parser._actions if isinstance(action, argparse._SubParsersAction)
+    ]
+
+    # Iterate through subparsers and get the one for the specific service
+    for subparsers_action in subparsers_actions:
+        # Get the service-specific parser
+        service_parser = subparsers_action.choices.get(service_name)
+        if service_parser:
+            # Extract the arguments and their defaults from the service-specific parser
+            service_args = {}
+            for action in service_parser._actions:
+                if action.dest != 'help':
+                    service_args[action.dest] = action.default
+            return service_args
+
+    return {}
+
+
+def ask_yes_no(
+        question: str,
+        default: Optional[bool] = None,
+        console: Console = None,
+        yes_color: str = "green",
+        no_color: str = "red",
+        dim_color: str = "dim white",
+        emoji: bool = True
+) -> bool:
+    """
+    Asks a yes/no question using Rich, placing the default option first and highlighting it.
+
+    Args:
+        question (str): The question to ask the user.
+        default (Optional[bool]): The default answer if the user just presses Enter.
+                                   If None, user must explicitly answer.
+        console (Console, optional): Rich Console object for printing.
+        yes_color (str, optional): Color for the "Yes" option. Defaults to "green".
+        no_color (str, optional): Color for the "No" option. Defaults to "red".
+        dim_color (str, optional): Color for dimmed text. Defaults to "dim white".
+        emoji (bool, optional): If True, includes emojis in the prompt.
+
+    Returns:
+        bool: True if the user answers yes, False otherwise.
+    """
+    console = console or Console()
+
+    # Define Yes/No options with emojis and colors
+    yes_option = f"{'✅ ' if emoji else ''}[bold {yes_color}]Yes[/bold {yes_color}]"
+    no_option = f"{'❌ ' if emoji else ''}[bold {no_color}]No[/bold {no_color}]"
+    dimmed_yes = f"{' ' if emoji else ''}[{dim_color}]Yes[/{dim_color}]"
+    dimmed_no = f"{' ' if emoji else ''}[{dim_color}]No[/{dim_color}]"
+
+    # Adjust prompt based on default value
+    if default is True:
+        prompt = f"{question} ({yes_option}/{dimmed_no}): "
+    elif default is False:
+        prompt = f"{question} ({no_option}/{dimmed_yes}): "
+    else:
+        prompt = f"{question} ({dimmed_yes}/{dimmed_no}): "
+
+    while True:
+        response = console.input(prompt).strip().lower()
+
+        if not response:  # User pressed Enter
+            if default is None:
+                console.print("❌ [bold red]You must explicitly answer 'yes' or 'no'.[/bold red]")
+                continue
+            return default
+        elif response in ['yes', 'y']:
+            return True
+        elif response in ['no', 'n']:
+            return False
+        else:
+            console.print("❌ [bold red]Invalid input.[/bold red] Please enter 'yes', 'no', or press Enter.")
+
