@@ -1,4 +1,6 @@
 from .file import (
+    NullByteRemover,
+    Tail,
     check_file_overwrite,
     get_file_path,
     get_file_extension,
@@ -9,6 +11,7 @@ from .file import (
     is_file,
     is_directory,
     is_json,
+    is_json_file,
     open_json,
     open_file,
     open_yaml_file,
@@ -44,11 +47,16 @@ from .color_print import (
     pretty_json,
     print_aligned_text,
     print_var,
+    print_var2,
+    create_kv_table,
+    print_kv,
+    print_grid,
     debug_logging,
     align_text,
     print_progress_bar,
     get_colorful_object,
     syntax_highlight,
     ProgressTime,
-    NoTraceBackException
+    NoTraceBackException,
+    get_color_by_threshold
 )
