@@ -1,9 +1,18 @@
 import copy
+import os
 import re
 import json
 import sys
+import inspect
+from rich.panel import Panel
 from pawnlib.config import pawn
 
+try:
+    from typing import Any, Union
+except ImportError:
+    from typing_extensions import Any, Union
+from pawnlib.typing.constants import const
+
 
 def is_json(s) -> bool:
     """
@@ -22,6 +31,8 @@ def is_json(s) -> bool:
             check.is_json('{"name": "John", "age": 30, "city": "New York",}')
             # >> False
     """
+    if not (isinstance(s, str) and (s.startswith('{') or s.startswith('['))):
+        return False
     try:
         json.loads(s)
     except ValueError:
@@ -52,11 +63,17 @@ def is_float(s) -> bool:
             # >> False
 
     """
+    if isinstance(s, bool):  # Exclude bool explicitly
+        return False
+    if isinstance(s, float):
+        return True
+    if isinstance(s, str):
+        s = s.strip()
     try:
-        float(s)
+        float_value = float(s)
+        return '.' in str(s) or 'e' in str(s).lower()  # Check if it has a decimal point or scientific notation
     except (TypeError, ValueError):
         return False
-    return True
 
 
 def is_int(s) -> bool:
@@ -85,11 +102,59 @@ def is_int(s) -> bool:
             # >> False
 
     """
+    if isinstance(s, bool):  # Exclude bool explicitly
+        return False
+    if isinstance(s, int):
+        return True
+    if isinstance(s, str):
+        s = s.strip()
     try:
-        int(s)
+        int_value = int(s)
+        return str(int_value) == str(s)  # Ensure the string representation matches the original input
+    except (TypeError, ValueError):
+        return False
+
+
+def is_number(s) -> bool:
+    """
+    Check if a value is an int or a float (number).
+
+    :param s: A value to check if it is a number
+    :type s: Any
+    :return: True if the value is an int or float, False otherwise
+    :rtype: bool
+
+    Example:
+
+        .. code-block:: python
+
+            is_number(42)
+            # >> True
+
+            is_number(3.14)
+            # >> True
+
+            is_number("42")
+            # >> True
+
+            is_number("3.14")
+            # >> True
+
+            is_number("hello")
+            # >> False
+
+    """
+    if isinstance(s, bool):  # Exclude bool explicitly
+        return False
+    if isinstance(s, str):
+        s = s.strip()
+    if isinstance(s, (int, float)):  # Direct check for int or float
+        return True
+    try:
+        float(s)  # Try converting to a float
+        return True
     except (TypeError, ValueError):
         return False
-    return True
 
 
 def is_hex(s) -> bool:
@@ -110,6 +175,10 @@ def is_hex(s) -> bool:
             # >> False
 
     """
+    if not isinstance(s, str):
+        return False
+    if s.startswith(("0x", "0X")):
+        s = s[2:]  # Remove '0x' prefix for validation
     try:
         int(s, 16)
     except TypeError:
@@ -213,7 +282,8 @@ def is_valid_ipv4(ip):
 
     """
     pattern = re.compile(
-        r"^((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)\.){3}(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)$",
+        # r"^((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)\.){3}(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)$",
+        const.PATTERN_IP_ADDRESS,
         re.VERBOSE | re.IGNORECASE
     )
     return pattern.match(ip) is not None
@@ -269,11 +339,12 @@ def is_valid_ipv6(ip):
     return pattern.match(ip) is not None
 
 
-def is_valid_url(url):
+def is_valid_url(url, strict=True):
     """
     Check if the given url is valid.
 
     :param url: (str) url to check
+    :param strict: If False, URLs without a TLD (e.g., "http://example") are considered valid. Defaults to True.
     :return: (bool) True if valid, False otherwise
 
     Example:
@@ -293,18 +364,47 @@ def is_valid_url(url):
             # >> False
 
     """
-    if "http://" not in url and "https://" not in url:
-        url = f"http://{url}"
+    if not url:
+        return False
 
-    regex = re.compile(
-        r'^https?://'  # http:// or https://
-        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?|'  # domain...
-        r'localhost|'  # localhost...
-        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})' # ...or ip
-        r'(?::\d+)?'  # optional port
-        r'(?:/?|[/?]\S+)',
-        re.IGNORECASE)
-    return url is not None and regex.search(url)
+    if url and not (url.startswith("http://") or url.startswith("https://")):
+            url = f"http://{url}"
+
+    # regex = re.compile(
+    #     r'^https?://'  # http:// or https://
+    #     r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?|'  # domain...
+    #     r'localhost|'  # localhost...
+    #     r'(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(?:\.(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)){3}\b)' # ...or ip
+    #     r'(?::\d+)?'  # optional port
+    #     r'(?:/?|[/?]\S+)',
+    #     re.IGNORECASE)
+
+
+    if strict:
+        # Standard regex pattern requiring a TLD
+        regex_pattern = (
+            r'^https?://'
+            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?|'  # domain
+            r'localhost|'  # localhost
+            r'(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(?:\.(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)){3}\b)'  # IP 
+            r'(?::\d+)?'  # optional port
+            r'(?:/?|[/?]\S+)'  # optional path
+            )
+
+    else:
+        # Regex pattern that allows URLs without a TLD
+        regex_pattern = (
+            r'^https?://'  # http://  or https://
+            r'(?:[A-Z0-9]+(?:[A-Z0-9-]*[A-Z0-9])?\.)?'  #  domain
+            r'(?:[A-Z0-9]+(?:[A-Z0-9-]*[A-Z0-9]))'  # parse 
+            r'(?:\.[A-Z]{2,6})?'  # TLD (선택사항)
+            r'(?::\d+)?'  # optional port
+            r'(?:/?|[/?]\S+)?$'  # optional path
+            )
+
+    regex = re.compile(regex_pattern, re.IGNORECASE)
+
+    return bool(regex.search(url))
 
 
 def is_valid_private_key(text=None):
@@ -392,6 +492,34 @@ def is_valid_tx_hash(text=None):
     return False
 
 
+def is_valid_icon_keystore_file(keystore=None):
+    from pawnlib.typing.converter import flatten
+    if not isinstance(keystore, dict):
+        return False
+
+    required_keys = [
+        "address",
+        "crypto.cipher", "crypto.cipherparams.iv",
+        "crypto.ciphertext", "crypto.kdf",
+        "crypto.kdfparams.dklen",
+        "crypto.kdfparams.n",
+        "crypto.kdfparams.r",
+        "crypto.kdfparams.p",
+        "crypto.kdfparams.salt",
+        "crypto.mac",
+        "id", "version", "coinType",
+    ]
+
+    flatten_keystore = flatten(keystore)
+    missing_keys = [key for key in required_keys if key not in flatten_keystore]
+
+    if missing_keys:
+        missing_keys_str = ", ".join(missing_keys)
+        raise ValueError(f"<Invalid Keystore> Missing required key(s): {missing_keys_str}")
+
+    return True
+
+
 def list_depth(l):
     """
     Returns the depth count of a list.
@@ -522,6 +650,30 @@ def return_guess_type(value):
         return value
 
 
+def error_and_exit(message, title="Error Occurred", exit_code=1):
+    """
+    Print an error message with the caller's file name and line number, then exit the program.
+
+    :param message: The error message to display.
+    :param title: The error title to display.
+    :param exit_code: The exit code to use when terminating the program (default is 1).
+    """
+    caller_frame = inspect.stack()[1]
+    file_name = os.path.basename(caller_frame.filename)
+    line_number = caller_frame.lineno
+    error_message = f"[bold red]Error:[/bold red] {message}"
+    subtitle = f"{file_name}:{line_number}"
+    print("")
+    pawn.console.print(
+        Panel(
+            error_message, title=f"[bold red]{title} (-{exit_code})[/bold red]", expand=True, subtitle=subtitle,
+            padding=1,
+        )
+    )
+    print("")
+    sys.exit(exit_code)
+
+
 def sys_exit(message="", return_code=-1):
     """
     This function executes the sys.exit() method.
@@ -580,24 +732,34 @@ def is_include_list(target=None, include_list=[], ignore_case=True):
     return False
 
 
-def keys_exists(element, *keys):
+def _traverse_keys(element: Union[dict, list], keys: tuple) -> Any:
     """
-    Check if **keys** (nested) exists in `element` (dict).
-    You don't have to implement it like this.
+    Helper function to traverse nested dictionaries and lists using the provided keys.
 
-    [X] if response.get('json') and response['json'].get('result') and response['json']['result'].get('tx_hash'): \n
-    [O] if keys_exists(response, 'json', 'result', 'tx_hash'): \n
+    :param element: The dictionary or list to traverse.
+    :param keys: The keys or indices to traverse in the dictionary or list.
+    :return: The value if all keys/indices exist, else raises an exception.
+    """
+    current_element = element
+    for key in keys:
+        if isinstance(current_element, list):
+            key = int(key)  # Convert key to int if the current element is a list
+        current_element = current_element[key]
+    return current_element
 
-    :param element: dictionary value
-    :param keys: The keys you want to find in the dictionary.
-    :return:
+
+def keys_exists(element: dict, *keys: str) -> bool:
+    """
+    Check if **keys** (nested) exist in `element` (dict).
+
+    :param element: The dictionary to search.
+    :param keys: The keys to traverse in the dictionary.
+    :return: True if all keys exist, False otherwise.
 
     Example:
 
         .. code-block:: python
 
-            from pawnlib.typing.check import keys_exists
-
             dict_example = {
                 "name": "example",
                 "description": {
@@ -605,7 +767,6 @@ def keys_exists(element, *keys):
                     "description_3": "333",
                 },
                 "none_value_key": None,
-
             }
 
             keys_exists(dict_example, 'name', 'description')
@@ -616,23 +777,61 @@ def keys_exists(element, *keys):
 
             keys_exists(dict_example, 'name', 'none_key')
             # >> False
-
     """
-    not_defined = "__NOT_DEFINED__"
-    next_element = ""
-    for index, key in enumerate(keys):
-        if next_element:
-            _element = next_element
-        else:
-            _element = copy.deepcopy(element)
-        if isinstance(_element, dict):
-            if _element.get(key, not_defined) != not_defined:
-                next_element = _element.pop(key, not_defined)
-            else:
-                return False
-        else:
-            return False
-    return True
+    try:
+        _traverse_keys(element, keys)
+        return True
+    except (KeyError, TypeError, IndexError, ValueError):
+        return False
+
+
+def get_if_keys_exist(element: dict, *keys: str, default: Any = None) -> Any:
+    """
+    Retrieve the value from a nested dictionary if **keys** exists in `element`.
+
+    :param element: The dictionary to search.
+    :param keys: The keys to traverse in the dictionary.
+    :param default: The default value to return if the keys do not exist.
+    :return: The value if all keys exist, else the default value.
+
+    Example:
+
+        .. code-block:: python
+
+            dict_example = {
+                "name": "example",
+                "description": {
+                    "description_2": "222",
+                    "description_3": "333",
+                },
+                "none_value_key": None,
+                "nested_list": [{"key1": "value1"}, {"key2": "value2"}]
+            }
+
+            get_if_keys_exist(dict_example, 'name')
+            # >> 'example'
+
+            get_if_keys_exist(dict_example, 'description', 'description_2')
+            # >> '222'
+
+            get_if_keys_exist(dict_example, 'none_value_key')
+            # >> None
+
+            get_if_keys_exist(dict_example, 'name', 'none_key')
+            # >> None
+
+            get_if_keys_exist(dict_example, 'nested_list', '1', 'key2')
+            # >> 'value2'
+
+            get_if_keys_exist(dict_example, 'nested_list', '0', 'key1')
+            # >> 'value1'
+    """
+    try:
+        return _traverse_keys(element, keys)
+    except (KeyError, TypeError, IndexError, ValueError) as e:
+        key_path = ' -> '.join(keys)
+        pawn.console.debug(f"Error accessing key/index '{key_path}': {e}")
+        return default
 
 
 def detect_encoding(byte_data, default_encode="utf8"):
@@ -675,3 +874,64 @@ def detect_encoding(byte_data, default_encode="utf8"):
             pass
     return default_encode
 
+
+def check_key_and_type(data, key, expected_type):
+    """
+    Checks if a specific key exists in a dictionary and if its value is of the expected type.
+
+    :param data: The dictionary to check.
+    :param key: The key to check for in the dictionary.
+    :param expected_type: The expected type of the value (e.g., list, dict, etc.).
+    :return: True if the key exists and its value is of the expected type, False otherwise.
+
+    Examples:
+
+        .. code-block:: python
+
+            result = {
+                'res': [1, 2, 3],
+                'config': {'option': True},
+                'count': 10
+            }
+
+            # Check if 'res' exists and is a list
+            is_res_list = check_key_and_type(result, 'res', list)
+            print(is_res_list)  # Output: True
+
+            # Check if 'config' exists and is a dict
+            is_config_dict = check_key_and_type(result, 'config', dict)
+            print(is_config_dict)  # Output: True
+
+            # Check if 'count' exists and is a string
+            is_count_string = check_key_and_type(result, 'count', str)
+            print(is_count_string)  # Output: False
+
+            # Check if 'nonexistent_key' exists and is a list
+            is_nonexistent_list = check_key_and_type(result, 'nonexistent_key', list)
+            print(is_nonexistent_list)  # Output: False
+    """
+    if key in data and isinstance(data[key], expected_type):
+        return True
+    return False
+
+
+
+def get_procfs_path():
+    """
+    Determine the appropriate procfs path based on the environment variables
+    and file system structure.
+
+    Returns:
+        str: The path to the proc filesystem.
+    """
+    is_docker = os.environ.get("IS_DOCKER", "").lower() in ["true", "1", "yes"]
+    docker_proc_path = "/rootfs/proc"
+
+    if is_docker and os.path.exists(docker_proc_path):
+        procfs_path = docker_proc_path
+        pawn.console.log(f"Running in Docker mode on {procfs_path}")
+    else:
+        procfs_path = "/proc"
+        pawn.console.log(f"Running in Host mode on {procfs_path}")
+
+    return procfs_path
