@@ -5,9 +5,9 @@ import hashlib
 import base64
 from os import path
 from coincurve import PrivateKey, PublicKey
-from pawnlib.typing import check, date_utils, random_private_key, fill_required_data_arguments, is_hex, format_hex
+from pawnlib.typing import check, date_utils, random_private_key, fill_required_data_arguments, is_hex, format_hex, is_valid_icon_keystore_file
 from pawnlib.config import pawnlib_config as pawn
-from pawnlib.output import is_file, is_json, open_json, check_file_overwrite, NoTraceBackException
+from pawnlib.output import is_file, is_json_file, open_json, check_file_overwrite, NoTraceBackException, print_json, print_var
 from pawnlib.config import pawn,  NestedNamespace
 from pawnlib.input import PromptWithArgument, PrivateKeyValidator, StringCompareValidator, PrivateKeyOrJsonValidator
 import json
@@ -46,31 +46,34 @@ def guess_wallet_type(data):
     If the `data` parameter is a valid JSON object, it is considered a JSON wallet type.
     If the wallet type cannot be determined, None is returned.
 
+
     Example:
 
-    .. code-block:: python
+        .. code-block:: python
 
-        # Example 1: PrivateKey wallet type
-        wallet_data = "0x1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef"
-        wallet_type = guess_wallet_type(wallet_data)
-        # wallet_type = "private_key"
+            from pawnlib.utils.icx_signer import guess_wallet_type
 
-        # Example 2: JSON wallet type
-        wallet_data = '{"address": "0x1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef"}'
-        wallet_type = guess_wallet_type(wallet_data)
-        # wallet_type = "json"
+            #Example 1: PrivateKey wallet type
+            wallet_data = "0x1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef"
+            wallet_type = guess_wallet_type(wallet_data)
+            # wallet_type = "private_key"
 
-        # Example 3: Unknown wallet type
-        wallet_data = 12345
-        wallet_type = guess_wallet_type(wallet_data)
-        # wallet_type = None
+            #Example 2: JSON wallet type
+            wallet_data = '{"address": "0x1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef"}'
+            wallet_type = guess_wallet_type(wallet_data)
+            # wallet_type = "json"
+
+            #Example 3: Unknown wallet type
+            wallet_data = 12345
+            wallet_type = guess_wallet_type(wallet_data)
+            # wallet_type = None
 
-    :raises: None
     """
+
     if isinstance(data, str) and len(data) == 66 or len(data) == 64:
         pawn.console.log("Wallet type is PrivateKey")
         return "private_key"
-    elif is_json(data):
+    elif is_json_file(data):
         pawn.console.log("Wallet type is JSON")
         return "json"
     else:
@@ -89,7 +92,7 @@ class WalletCli:
             password=""
         )
         self._args = fill_required_data_arguments(required=required_args)
-        self._wallet = ""
+        self._wallet = {}
 
     def load(self):
         keystore_json = {}
@@ -147,7 +150,7 @@ class WalletCli:
                 type="list",
                 default="",
                 argument="keystore",
-            ).select()
+            ).fuzzy()
             _required_password = True
 
         elif load_type == "text":
@@ -160,13 +163,18 @@ class WalletCli:
 
         if _keystore:
             keystore = str(_keystore).strip()
-            if is_file(keystore) and is_json(keystore):
+
+            if is_file(keystore) and is_json_file(keystore):
                 pawn.console.debug(f"Found Keystore JSON file - {keystore}")
                 try:
-                    keystore_json = open_json(keystore)
-                except ValueError:
-                    pawn.console.log(f"[red]Invalid JSON file - {keystore}")
+                    keystore_json = open_json(keystore, encoding='utf-8-sig')
+                except Exception as e:
+                    pawn.console.log(f"[red]Invalid JSON file - {keystore}, Exception: {e}")
                 _required_password = True
+
+                if not is_valid_icon_keystore_file(keystore_json):
+                    raise ValueError(f"Invalid keystore file - {self._args.keystore}")
+
             elif is_hex(keystore):
                 if is_private_key(keystore):
                     pawn.console.log("Found Private key")
@@ -184,7 +192,7 @@ class WalletCli:
                         raise ValueError("Invalid JSON or Keystore text")
                     _required_password = True
                 except Exception as e:
-                    raise ValueError(f"[red][Error] cannot load - {e}")
+                    raise ValueError(f"[red][Error][/red] Failed to load file '{self._args.keystore}'. JSON parsing error at: {e}")
 
         if _required_password:
             _password = PromptWithArgument(
@@ -198,6 +206,7 @@ class WalletCli:
 
         if keystore_json:
             self._wallet = load_wallet_key(keystore_json, _password)
+
             if self._wallet:
                 self.print_wallet()
             else:
@@ -206,14 +215,28 @@ class WalletCli:
         return self._wallet
 
     def create(self, is_store_file=True):
-        PromptWithArgument(
-            message="Enter your private key (default: empty is random)",
-            type="input",
-            default="",
-            argument="private_key",
-            validate=PrivateKeyValidator(allow_none=True),
-            # verbose=0,
-        ).prompt()
+        self._handle_password_prompt()
+
+        if not self._args.private_key:
+            self._args.private_key = random_private_key()
+
+        self._wallet = load_wallet_key(self._args.private_key, password=self._args.password)
+        self.print_wallet()
+
+        if is_store_file:
+            self._save_wallet_to_file()
+        else:
+            self._generate_wallet(is_store_file=False)
+
+    def _handle_password_prompt(self):
+        if hasattr(self._args, "password") and not self._args.password:
+            PromptWithArgument(
+                message="Enter your private key (default: empty is random)",
+                type="input",
+                default="",
+                argument="private_key",
+                validate=PrivateKeyValidator(allow_none=True),
+            ).prompt()
 
         PromptWithArgument(
             message="Enter password for private key",
@@ -224,33 +247,36 @@ class WalletCli:
             validate=lambda result: len(result) >= 1,
         ).prompt()
 
-        if not self._args.private_key:
-            self._args.private_key = random_private_key()
+    def _save_wallet_to_file(self):
+        default_filename = f"{self._wallet.get('address')}_{date_utils.todaydate('ms_text')}.json"
+        PromptWithArgument(
+            message="Enter the name of JSON file to be saved.",
+            default=default_filename,
+            argument="keystore",
+            invalid_message="Requires at least one character.",
+            validate=lambda result: len(result) >= 1,
+        ).prompt()
 
-        self._wallet = load_wallet_key(self._args.private_key, password=self._args.password)
-        self.print_wallet()
+        if check_file_overwrite(filename=self._args.keystore):
+            self._generate_wallet()
 
-        if is_store_file:
-            default_filename = f"{self._wallet.get('address')}_{date_utils.todaydate('ms_text')}.json"
-            PromptWithArgument(
-                message="Enter the name of JSON file to be saved.",
-                default=default_filename,
-                argument="keystore",
-                invalid_message="Requires at least one character.",
-                validate=lambda result: len(result) >= 1,
-            ).prompt()
-            if check_file_overwrite(filename=self._args.keystore):
-                try:
-                    wallet = generate_wallet(
-                        file_path=self._args.keystore,
-                        password=self._args.password,
-                        overwrite=False,
-                        private_key=self._args.private_key,
-                        expected_address=self._wallet.get('address'),
-                    )
-                    pawn.console.log(f"Generate Wallet - {wallet.get_hx_address()} to '{self._args.keystore}'")
-                except Exception as e:
-                    pawn.console.log(f"[red][ERROR] Generate wallet - {e}")
+    def _generate_wallet(self, is_store_file=True):
+        try:
+            wallet = generate_wallet(
+                file_path=self._args.keystore,
+                password=self._args.password,
+                overwrite=False,
+                private_key=self._args.private_key,
+                expected_address=self._wallet.get('address'),
+                is_store_file=is_store_file
+            )
+            if is_store_file:
+                pawn.console.log(f"Generate Wallet - {wallet.get_hx_address()} to '{self._args.keystore}'")
+            else:
+                pawn.console.print(" 🔑 keystore content")
+                print_json(wallet.key_store_content)
+        except Exception as e:
+            pawn.console.log(f"[red][ERROR] Generate wallet - {e}")
 
     def print_wallet(self):
         if self._wallet:
@@ -275,11 +301,16 @@ def store_keystore_file_on_the_path(file_path, json_string, overwrite=False):
         f.write(json_string)
 
 
-def generate_wallet(file_path=None, password=None, overwrite=False, private_key=None, expected_address=None):
+def generate_wallet(file_path=None, password=None, overwrite=False, private_key=None, expected_address=None, is_store_file=True):
     singer = IcxSigner(data=private_key)
     if not file_path:
         file_path = f"{singer.get_hx_address()}_{date_utils.todaydate('ms_text')}.json"
-    singer.store(file_path, password, overwrite, expected_address=expected_address)
+
+    if is_store_file:
+        singer.store(file_path, password, overwrite, expected_address=expected_address)
+    else:
+        singer.create_key_store_content(password)
+
     return singer
 
 
@@ -449,6 +480,8 @@ class IcxSigner(object):
         """
         self._private_key_hex = None
         self._private_key_bytes = None
+        self.key_store_content = {}
+
         if data:
             self._check_private_key(data)
 
@@ -531,24 +564,41 @@ class IcxSigner(object):
         signature = privkey.sign(msg_hash, hasher=None)
         return signature
 
-    def store(self, file_path: str, password: str, overwrite: bool = False, expected_address: str = None):
+    def create_key_store_content(self, password: str, iterations=16384, kdf="scrypt"):
         try:
-            key_store_contents = create_keyfile_json(
+            self.key_store_content = {}
+            self.key_store_content = create_keyfile_json(
                 self.get_privkey_bytes(),
                 bytes(password, 'utf-8'),
+                iterations=iterations,
+                kdf=kdf
+            )
+            self.key_store_content['address'] = self.get_hx_address()
+            self.key_store_content['coinType'] = 'icx'
+            return self.key_store_content
+        except Exception as e:
+            raise ValueError(e)
+
+    def store(self, file_path: str, password: str, overwrite: bool = False, expected_address: str = None):
+        try:
+            # key_store_contents = create_keyfile_json(
+            #     self.get_privkey_bytes(),
+            #     bytes(password, 'utf-8'),
+            #     iterations=16384,
+            #     kdf="scrypt"
+            # )
+
+            self.key_store_content = self.create_key_store_content(
+                password,
                 iterations=16384,
                 kdf="scrypt"
             )
-            key_store_contents['address'] = self.get_hx_address()
-            key_store_contents['coinType'] = 'icx'
-
             # validate the  contents of a keystore file.
             if expected_address and expected_address != self.get_hx_address():
                 raise ValueError(f"Not expected address => expected({expected_address}) != real({self.get_hx_address()})")
 
-            import json
-            if key_store_contents:
-                json_string_keystore_data = json.dumps(key_store_contents)
+            if self.key_store_content:
+                json_string_keystore_data = json.dumps(self.key_store_content)
                 store_keystore_file_on_the_path(file_path, json_string_keystore_data, overwrite)
                 pawn.console.debug(f"Stored Wallet. Address: {self.get_hx_address()}, File path: {file_path}")
         except FileExistsError:
